/***************************************************************************
 *   Copyright (C) 2006 by George Kiagiadakis   *
 *   gkiagia@users.sourceforge.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QFileInfo>
#include <QMessageBox>
#include <QProcess>
#include <QStringList>
#include <QVariant>
#include <QModelIndex>
#include <QLabel>
#include "macros.h"
#include "confighandler.h"
#include "createdialog.h"

ConfigHandler *ConfigHandler::self = NULL;

ConfigHandler::ConfigHandler(QObject *parent)
	:QObject(parent)
{
	ConfigHandler::self = this;
	if ( !initializeCfgDir() )
	{
		QMessageBox::critical(0, "Error", QString("There was a problem using %1 as a configuration directory.\nPress OK to use the default.").arg( QtWine::settings->value("general/config_dir", DEFAULT_CONFIG_PATH ).toString() ), "OK");
		QtWine::settings->setValue("general/config_dir", DEFAULT_CONFIG_PATH);
		QtWine::settings->sync();
		if ( !initializeCfgDir() )
		{
			QMessageBox::critical(0, "Error", QString("There was a problem using %1 as a configuration directory.\nQtWine will now exit.").arg( QtWine::settings->value("general/config_dir", DEFAULT_CONFIG_PATH ).toString() ), "OK");
			exit(1);
		}
	}
	populateCfgList();
	cfgModel = new ConfigModel(&cfgList, this);
}

ConfigHandler::~ConfigHandler() {}

bool ConfigHandler::initializeCfgDir()
{
	if ( !cfgDir.cd( QtWine::settings->value("general/config_dir", DEFAULT_CONFIG_PATH).toString() ) ) // if cfg_dir doesn't exist...
	{
		if ( !cfgDir.mkdir( QtWine::settings->value("general/config_dir", DEFAULT_CONFIG_PATH).toString() ) ) // ...create it!
		{
			QMessageBox::critical(0, "Error", QString("Could not create directory %1").arg( QtWine::settings->value("general/config_dir", DEFAULT_CONFIG_PATH).toString() ), "OK");
			return FALSE;
		}
		return cfgDir.cd( QtWine::settings->value("general/config_dir", DEFAULT_CONFIG_PATH).toString() ); // return TRUE. Directory is writable (because it was created without problems) so there is no need to go to the next if.
		// if ever this returns FALSE, directory is not readable so FALSE must be returned.
	}

	QFile f( cfgDir.absolutePath()+="qtwine.txt" );
	if ( !f.open(QIODevice::ReadWrite) ) // works. Checks if directory is writable
	{
		QMessageBox::critical(0, "Error", QString("Directory %1 is not writable.").arg( QtWine::settings->value("general/config_dir", DEFAULT_CONFIG_PATH).toString() ), "OK");
		return FALSE;
	}
	else
	{
		f.close();
		f.remove();
	}

	return TRUE;
}

void ConfigHandler::connectListView(QListView *ptr)
{
	listView_list.append(ptr);
	ptr->setModel(cfgModel);
	ptr->selectionModel()->select( cfgModel->index(0,0) , QItemSelectionModel::SelectCurrent ); // select the first config
	ptr->selectionModel()->setCurrentIndex( cfgModel->index(0,0) , QItemSelectionModel::SelectCurrent ); // make it current
	//connect(list, SIGNAL(clicked(const QModelIndex & index)), this, SLOT(userClick(const QModelIndex & index)) ); // TODO new features...
}

void ConfigHandler::disconnectListView(QListView *ptr)
{
	listView_list.removeAt( listView_list.indexOf(ptr) );
}

QString ConfigHandler::getCurrentCfgName(QListView *listView)
{
	QModelIndex currentItem = listView->selectionModel()->currentIndex();
	return cfgModel->data(currentItem, Qt::DisplayRole).toString();
}

QString ConfigHandler::getCurrentCfgPath(QListView *listView)
{
	QModelIndex currentItem = listView->selectionModel()->currentIndex();
	return cfgModel->data(currentItem, Qt::UserRole).toString();
}

QString ConfigHandler::getCfgPath(QString cfgName)
{
	foreach(Configuration *cfg, cfgList)
	{
		if ( cfgName == cfg->name() )
			return cfg->absolutePath();
	}
	return QString();
}

QString ConfigHandler::generateCfgPath(QString newCfgName)
{
	if (newCfgName == "Wine Default")
		return QDir::homePath() += "/.wine";
	else
		return cfgDir.absolutePath() + '/' + newCfgName;
}

void ConfigHandler::removeCurrentCfg(QListView *listView)
{
	QModelIndex currentItem = listView->selectionModel()->currentIndex();
	cfgModel->removeRows( currentItem.row(), 1 );

	foreach(QListView *lv, listView_list) // select the first config again because removeRows() causes model reset()
	{
		lv->selectionModel()->select( cfgModel->index(0,0) , QItemSelectionModel::Select );
		lv->selectionModel()->setCurrentIndex( cfgModel->index(0,0) , QItemSelectionModel::SelectCurrent );
	}
}

void ConfigHandler::changeCfgDir(QString newDir)
{
	QString oldDir = QtWine::settings->value("general/config_dir", DEFAULT_CONFIG_PATH ).toString();
	QtWine::settings->setValue("general/config_dir", newDir);
	QtWine::settings->sync();
	if ( !initializeCfgDir() )
	{
		QMessageBox::critical(0, "Error", QString("There was a problem using %1 as a configuration directory.\nPress OK to use the previously used one.").arg( QtWine::settings->value("general/config_dir").toString() ), "OK");
		QtWine::settings->setValue("general/config_dir", oldDir);
		QtWine::settings->sync();
		if ( !initializeCfgDir() )
		{
			QMessageBox::critical(0, "Error", QString("There was a problem using %1 as a configuration directory.\nQtWine will now exit.").arg( QtWine::settings->value("general/config_dir").toString() ), "OK");
			exit(1);
		}
	}
	delete cfgModel;
	populateCfgList();
	cfgModel = new ConfigModel(&cfgList, this);
	foreach(QListView *lv, listView_list)
	{
		lv->setModel(cfgModel);
		lv->selectionModel()->select( cfgModel->index(0,0) , QItemSelectionModel::Select );
		lv->selectionModel()->setCurrentIndex( cfgModel->index(0,0) , QItemSelectionModel::SelectCurrent );
	}
}

// CREATE FUNCTIONS

bool ConfigHandler::create(QString newCfgPath, QString newCfgName)
{
	QProcess wineprefixcreate;
	wineprefixcreate.setReadChannelMode(QProcess::MergedChannels); // FIXME does mergedchannels work in 4.1.4 ?
	wineprefixcreate.start("wineprefixcreate", QStringList() << "--prefix" << newCfgPath);

	
	wineprefixcreate.waitForFinished(120000); // 2 min


	if ( wineprefixcreate.exitStatus() == QProcess::NormalExit ) {
		if ( wineprefixcreate.exitCode() == 0 ) {
			Configuration *c = new Configuration( newCfgPath, newCfgName );
			insertConfiguration(c);
			QMessageBox::information(0, "Success", "Your new wine configuration was created successfully.\nwineprefixcreate exit code = 0", "OK");
			return TRUE;
		} else {
			QMessageBox::information(0, "Failure", QString("Your new wine configuration was NOT created successfully.\nwineprefixcreate exit code = %1\n\nStandard output / Standard error:\n%2").arg(wineprefixcreate.exitCode()).arg( QString(wineprefixcreate.readAllStandardOutput()) ), "OK");
		}
	} else {
		QMessageBox::information(0, "Failure", QString("Your new wine configuration was NOT created successfully.\nwineprefixcreate crashed\n\nStandard output / Standard error:\n%1").arg( QString(wineprefixcreate.readAllStandardOutput()) ), "OK");
	}

	// probably TRUE was not returned so there is an error
	// we have to remove the new directory if it was created
	if ( QDir(newCfgPath).exists() ) {
		Configuration c( newCfgPath, "To be removed" );
		c.rmSelf();
	}
	return FALSE;
}

bool ConfigHandler::duplicate(QString sourceCfgName, QString newCfgName) // TODO update qtwine.ini and fixdriveC
{
	QString sourceCfgPath = getCfgPath(sourceCfgName);

	QProcess cp;
	cp.setReadChannelMode(QProcess::MergedChannels);
	cp.start("cp", QStringList() << "-fRpP" << sourceCfgPath << cfgDir.filePath(newCfgName) );

	cp.waitForFinished(-1);

	if ( cp.exitStatus() == QProcess::NormalExit ) {
		if ( cp.exitCode() == 0 ) {
			Configuration *c = new Configuration( generateCfgPath(newCfgName), newCfgName );
			c->personalize();
			insertConfiguration(c);
			QMessageBox::information(0, "Success", "Configuration was copied successfully.\ncp exit code = 0", "OK");
			return TRUE;
		} else {
			QMessageBox::information(0, "Failure", QString("Configuration was NOT copied successfully.\ncp exit code = %1\n\nStandard output / Standard error:\n%2").arg(cp.exitCode()).arg( QString(cp.readAllStandardOutput()) ), "OK");
		}
	} else {
		QMessageBox::information(0, "Failure", QString("Configuration was NOT copied successfully.\ncp crashed\n\nStandard output / Standard error:\n%1").arg( QString(cp.readAllStandardOutput()) ), "OK");
	}

	// probably TRUE was not returned so there is an error
	// we have to remove the new directory if it was created
	if (cfgDir.exists(newCfgName)) {
		Configuration c( cfgDir.filePath(newCfgName), "To be removed" );
		c.rmSelf();
	}
	return FALSE;
}

void ConfigHandler::insertConfiguration(Configuration *cfg) const
{
	cfgModel->insertConfiguration(cfg);
}

// CONFIGURATION LIST RELATED

bool ConfigHandler::configurationExists(QString cfgName)
{
	foreach(Configuration *cfg, cfgList)
	{
		if ( cfgName == cfg->name() )
			return TRUE;
	}
	return FALSE;
}

void ConfigHandler::populateCfgList() // This function reads configurations from disk.
{
	QStringList names;
	QString cfgPath = cfgDir.absolutePath() + '/';
	cfgList.clear();
	Configuration *a;

	names = cfgDir.entryList( QDir::Dirs|QDir::NoDotAndDotDot, QDir::Name );
	foreach( QString cfg, names)
	{
		a = new Configuration(cfgPath+cfg, cfg);
		cfgList.append( a );
	}

	if ( QtWine::settings->value("general/use_wine_default_cfg", TRUE).toBool() )
	{
		QDir d = QDir::home();
		if ( !d.cd(".wine") )
		{
			QMessageBox::information( 0, "Error", QDir::homePath()+="/.wine doesn't exist.\nWine Default Configuration will be disabled.\n\nTo use it, create it by running \"wineprefixcreate\"\nand then enable it from QtWine options.", "OK" );
			QtWine::settings->setValue("general/use_wine_default_cfg", FALSE);
		}
		else
		{
			a = new Configuration(d.absolutePath(), "Wine Default");
			cfgList.prepend( a );
		}
	}
}


